import 'dart:convert';

import 'package:mobx/mobx.dart';

import 'package:models_base/utils.dart';

abstract class ArticleAbstract {
  final int lineId;
  int get productId => lineId;
  final int id;
  String fullName;
  int price;
  int cost;
  double weight;
  int? articleCode;
  String? photo;
  @observable
  bool status;
  final DateTime? creationDate;
  final DateTime? updateDate;

  ArticleAbstract({
    required this.lineId,
    required this.id,
    required this.fullName,
    this.price = 0,
    this.cost = 0,
    this.weight = 1.0,
    required this.creationDate,
    required this.updateDate,
    this.articleCode,
    this.photo,
    required this.status,
  });

  String nameWithoutProductPart(String productTitle) {
    String name;
    if (productTitle.length < fullName.length &&
        fullName.contains(productTitle)) {
      name = fullName.substring(productTitle.length);
    } else {
      name = fullName;
    }
    return name.trim();
  }

  @override
  String toString() {
    return """
Article(
  lineId: $lineId,
  id: $id,
  fullName: '$fullName',
  price: $price,
  creationDate: $creationDate,
  updateDate: $updateDate,
  cost: $cost,
  weight: $weight,
  articleCode: $articleCode,
  photo: $photo,
  status: $status,
)
""";
  }

  Map<String, dynamic> toMap() {
    return {
      'lineId': lineId,
      'id': id,
      'fullName': fullName,
      'price': price,
      'cost': cost,
      'weight': weight,
      'articleCode': articleCode ?? 0,
      'photo': photo ?? '',
      'creationDate': creationDate?.toIso8601String(),
      'updateDate': updateDate?.toIso8601String(),
      'status': status,
    };
  }

  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is ArticleAbstract &&
        other.lineId == lineId &&
        other.id == id &&
        other.fullName == fullName &&
        other.price == price &&
        other.cost == cost &&
        other.weight == weight &&
        other.articleCode == articleCode &&
        other.photo == photo &&
        other.creationDate == creationDate &&
        other.updateDate == updateDate &&
        other.status == status;
  }

  @override
  int get hashCode {
    return lineId.hashCode ^
        id.hashCode ^
        fullName.hashCode ^
        price.hashCode ^
        cost.hashCode ^
        weight.hashCode ^
        articleCode.hashCode ^
        photo.hashCode ^
        creationDate.hashCode ^
        updateDate.hashCode ^
        status.hashCode;
  }
}

class ArticleDummy extends ArticleAbstract {
  ArticleDummy()
      : super(
          lineId: 1,
          id: 1,
          fullName: 'frometon forever',
          price: 1000,
          cost: 500,
          weight: 1,
          articleCode: 1,
          photo: 'photo',
          creationDate: WeebiDates.defaultDate,
          updateDate: WeebiDates.defaultDate,
          status: true,
        );
}
