import 'dart:convert';

import 'package:collection/collection.dart';

import 'package:models_base/src/base/article_base.dart';
import 'package:models_base/src/base/lot_base.dart';

abstract class ItemAbstract<A extends ArticleAbstract, L extends LotAbstract> {
  final A article;
  List<L>? lots;
  double quantity;

  ItemAbstract(this.article, this.lots, this.quantity);

  @override
  String toString() => "Item($article,$lots,$quantity)";

  Map<String, dynamic> toMap() {
    return {
      'article': article.toMap(),
      'lots': lots?.map((x) => x.toMap()).toList(),
      'quantity': quantity,
    };
  }

  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is ItemAbstract<A, L> &&
        other.article == article &&
        listEquals(other.lots, lots) &&
        other.quantity == quantity;
  }

  @override
  int get hashCode => article.hashCode ^ lots.hashCode ^ quantity.hashCode;
}

class ItemDummy extends ItemAbstract<ArticleDummy, LotDummy> {
  ItemDummy()
      : super(
          ArticleDummy(),
          [LotDummy()],
          2.0,
        );
}
