import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:mobx/mobx.dart';

import 'package:models_base/common.dart' show StockUnit;
import 'package:models_base/src/base/article_base.dart';
import 'package:models_base/utils.dart';

abstract class LineArticleAbstract<T extends ArticleAbstract> {
  final int id;
  String title;
  StockUnit stockUnit;
  String? photo;
  int? barcode;
  DateTime? statusUpdateDate;
  List<T> articles;
  final List<String>? categories; // line != categories
  final DateTime? creationDate;
  DateTime? updateDate;
  @observable
  bool status;

  LineArticleAbstract({
    required this.id,
    required this.title,
    required this.articles,
    this.stockUnit = StockUnit.unit,
    this.photo,
    this.categories,
    this.barcode,
    this.statusUpdateDate,
    required this.creationDate,
    required this.updateDate,
    required this.status,
  });

  // factory Product.fromJson(Map<String, dynamic> json) =>
  //     _$ProductFromJson(json);

  Map<String, dynamic> toMap() {
    return {
      'id': id,
      'title': title,
      'stockUnit': stockUnit.toString(),
      'photo': photo ?? '',
      'barcode': barcode ?? 0,
      'status': status,
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'articles': articles.map((x) => x.toMap()).toList(),
      'creationDate':
          creationDate == null ? '' : creationDate?.toIso8601String(),
      'updateDate': updateDate == null ? '' : updateDate?.toIso8601String(),
      'categories': categories?.map((e) => e).toList(),
    };
  }

  String toJson() => json.encode(toMap());

  @override
  String toString() {
    return '''
ArticleLine(
  id: $id,
  title: $title,
  stockUnit: $stockUnit,
  photo: $photo,
  barcode: $barcode,
  statusUpdateDate: $statusUpdateDate,
  updateDate: $updateDate,
  creationDate: $creationDate,
  status: $status,
  articles: $articles)  
    ''';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is LineArticleAbstract<T> &&
        other.id == id &&
        other.title == title &&
        other.stockUnit == stockUnit &&
        other.photo == photo &&
        other.barcode == barcode &&
        other.status == status &&
        other.statusUpdateDate == statusUpdateDate &&
        listEquals(other.articles, articles) &&
        other.creationDate == creationDate &&
        other.updateDate == updateDate;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        title.hashCode ^
        stockUnit.hashCode ^
        photo.hashCode ^
        barcode.hashCode ^
        status.hashCode ^
        statusUpdateDate.hashCode ^
        articles.hashCode ^
        creationDate.hashCode ^
        updateDate.hashCode;
  }
}

class LineArticleDummy extends LineArticleAbstract {
  LineArticleDummy()
      : super(
          id: 1,
          title: 'frometon',
          status: true,
          articles: [ArticleDummy()],
          stockUnit: StockUnit.unit,
          photo: 'photo',
          categories: ['categories'],
          barcode: 1,
          creationDate: WeebiDates.defaultDate,
          updateDate: WeebiDates.defaultDate,
          statusUpdateDate: WeebiDates.defaultDate,
        );
}
