import 'dart:convert';

abstract class LotAbstract {
  final int lineId;
  int get productId => lineId;
  final int articleId;
  final int id;
  final bool status;
  final int proxyLineId;
  final int proxyArticleId;

  const LotAbstract({
    required this.lineId,
    required this.articleId,
    required this.id,
    required this.proxyLineId,
    required this.proxyArticleId,
    this.status = true,
  });

  String get text {
    return 'Lot $lineId.$articleId.$id';
  }

  @override
  String toString() {
    return '''
Lot(
    lineId: $lineId,
    articleId: $articleId,
    id: $id,
    'proxyArticleId': $proxyArticleId,
    'proxyLineId': $proxyLineId,
    status: $status,
''';
  }

  Map<String, dynamic> toMap() {
    return {
      'lineId': lineId,
      'articleId': articleId,
      'id': id,
      'proxyArticleId': proxyArticleId,
      'proxyLineId': proxyLineId,
      'status': status,
    };
  }

  String toJson() => json.encode(toMap());

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is LotAbstract &&
        other.lineId == lineId &&
        other.articleId == articleId &&
        other.id == id &&
        other.proxyLineId == proxyLineId &&
        other.proxyArticleId == proxyArticleId &&
        other.status == status;
  }

  @override
  int get hashCode {
    return lineId.hashCode ^
        articleId.hashCode ^
        id.hashCode ^
        status.hashCode ^
        proxyArticleId.hashCode & proxyLineId.hashCode;
  }
}

class LotDummy extends LotAbstract {
  LotDummy()
      : super(
          lineId: 1,
          articleId: 1,
          id: 1,
          proxyLineId: 1,
          proxyArticleId: 1,
          status: true,
        );
}
